/*
   (c) Copyright 2001-2002  convergence integrated media GmbH.
   (c) Copyright 2002-2005  convergence GmbH.

   Written by Denis Oliver Kropp <dok@directfb.org>
              
   This file is subject to the terms and conditions of the MIT License:

   Permission is hereby granted, free of charge, to any person
   obtaining a copy of this software and associated documentation
   files (the "Software"), to deal in the Software without restriction,
   including without limitation the rights to use, copy, modify, merge,
   publish, distribute, sublicense, and/or sell copies of the Software,
   and to permit persons to whom the Software is furnished to do so,
   subject to the following conditions:

   The above copyright notice and this permission notice shall be
   included in all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
   EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
   MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
   IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
   CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
   TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
   SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#include <config.h>

#include  <stdio.h>
#include  <sys/types.h>
#include <stddef.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include <lite/lite.h>
#include <lite/window.h>
#include <lite/util.h>

#include <leck/textbutton.h>
#include <leck/button.h>
#include <leck/image.h>
#include <leck/label.h>
#include <leck/textline.h>


static void execute(char *text, void *ctx)
{
     int   i   = 0;
     int   len = strlen(text);
     char *buf, *p, *args[len+1];
     int cpid;

     if (len < 1)
          return;

     buf = p = strdup( text );


     while (*p) {
          if (*p == ' ') {
               *p++ = 0;
               continue;
          }

          args[i++] = p++;

          while (*p && *p != ' ')
               p++;
     }

     if (i < 1)
          return;

     /* destroy the window with all this children and resources */
     lite_destroy_window( LITE_WINDOW(ctx) );

     /* deinitialize */
     lite_close();

     args[i] = NULL;

     cpid = fork();
     if(cpid == 0) {
         execvp( args[0], args );
         perror( "execvp" );
         exit(-1);
     } else if (cpid<0) {
         perror( "fork" );
     } else {
         //wait();
     }

}


static void button_pressed( LiteTextButton *button, void *ctx )
{
execute("dfbterm",ctx);
}

static void button2_pressed ( LiteTextButton *button, void *ctx )
{
execute("/lbin/ts7390-init",ctx);
}

static void button3_pressed ( LiteTextButton *button, void *ctx )
{
execute("/usr/gtkdfb/bin/gtk-demo",ctx);
}

static void button4_pressed( LiteTextButton *button, void *ctx )
{
execute("df_dok",ctx);
}

static void button5_pressed ( LiteTextButton *button, void *ctx )
{
execute("df_neo",ctx);
}

static void button6_pressed ( LiteTextButton *button, void *ctx )
{
execute("df_andi",ctx);
}

static void button7_pressed ( LiteTextButton *button, void *ctx )
{

execute("/bin/sh /lbin/start-full-debian",ctx);
}

static void button8_pressed ( LiteTextButton *button, void *ctx )
{
execute("reboot",ctx);
}

static void button9_pressed ( LiteTextButton *button, void *ctx )
{
lite_close_window( LITE_WINDOW(ctx) );
}


DFBColor red = { r:255,g:0,b:0,a:0 };
DFBColor white = { r:255,g:255,b:125,a:0 };
DFBColor blue = { r:0,g:0,b:255,a:0 };
DFBColor green = { r:0,g:155,b:0,a:0 }; 

int
main (int argc, char *argv[])
{
     LiteTextButton *button, *button2, *button3;
     LiteTextButton *button4, *button5, *button6;
     LiteTextButton *button7, *button8, *button9;
     LiteTextButtonTheme *textButtonTheme1;
     LiteTextButtonTheme *textButtonTheme2;
     LiteTextButtonTheme *textButtonTheme3;

     LiteLabel *label;
     LiteWindow     *window;
     DFBRectangle    rect;
     DFBResult       res;
     LiteFont *fontbtn; 

     /* initialize */
     if (lite_open( &argc, &argv ))
          return 1;

     /* create a window */
     rect.x = LITE_CENTER_HORIZONTALLY;
     rect.y = LITE_CENTER_VERTICALLY;
     rect.w = 800;
     rect.h = 480;

     res = lite_new_window( NULL, 
                            &rect,
                            DWCAPS_ALPHACHANNEL, 
                            liteDefaultWindowTheme,
                            "Technologic Systems",
                            &window );

     lite_get_font("Arial", LITE_FONT_PLAIN, 18, 0, &fontbtn);

     res = lite_new_text_button_theme(
          EXAMPLESDATADIR "/textbuttonbgnd.png",
          &textButtonTheme1);
     textButtonTheme1->font = fontbtn;
     textButtonTheme1->font_color = green;

     res = lite_new_text_button_theme(
          EXAMPLESDATADIR "/textbuttonbgnd.png",
          &textButtonTheme2);
     textButtonTheme2->font = fontbtn;
     textButtonTheme2->font_color = blue;

     res = lite_new_text_button_theme(
          EXAMPLESDATADIR "/textbuttonbgnd.png",
          &textButtonTheme3);
     textButtonTheme3->font = fontbtn;
     textButtonTheme3->font_color = red;

     if( res != DFB_OK )
          return res;

     rect.x = 50; rect.y = 30; rect.w = 700;
     res = lite_new_label(LITE_BOX(window), &rect, liteNoLabelTheme, 24, &label );
     lite_set_label_text( label, "TECHNOLOGIC SYSTEMS INC  -  DIRECTFB FASTGUI DEMO" );

     /* setup the button */
     rect.x = 50; rect.y = 100; rect.w = 200; rect.h = 90;
     res = lite_new_text_button(LITE_BOX(window), &rect, "TERMINAL", textButtonTheme1, &button);
     lite_on_text_button_press( button, button_pressed, window );

     rect.x = 300; rect.y = 100; rect.w = 200; rect.h = 90;
     res = lite_new_text_button(LITE_BOX(window), &rect, "START XORG", textButtonTheme1, &button2);
     lite_on_text_button_press( button2, button2_pressed, window );

     rect.x = 550; rect.y = 100; rect.w = 200; rect.h = 90;
     res = lite_new_text_button(LITE_BOX(window), &rect, "GTK DEMO", textButtonTheme1, &button3);
     lite_on_text_button_press( button3, button3_pressed, window );

     rect.x = 50; rect.y = 220; rect.w = 200; rect.h = 90;
     res = lite_new_text_button(LITE_BOX(window), &rect, "DFB BENCHMARK", textButtonTheme2, &button4);
     lite_on_text_button_press( button4, button4_pressed, window );

     rect.x = 300; rect.y = 220; rect.w = 200; rect.h = 90;
     res = lite_new_text_button(LITE_BOX(window), &rect, "DFB NEO SAMPLE", textButtonTheme2, &button5);
     lite_on_text_button_press( button5, button5_pressed, window );

     rect.x = 550; rect.y = 220; rect.w = 200; rect.h = 90;
     res = lite_new_text_button(LITE_BOX(window), &rect, "DFB ANDI SAMPLE", textButtonTheme2, &button6);
     lite_on_text_button_press( button6, button6_pressed, window );

     rect.x = 50; rect.y = 340; rect.w = 200; rect.h = 90;
     res = lite_new_text_button(LITE_BOX(window), &rect, "FULL DEBIAN", textButtonTheme3, &button7);
     lite_on_text_button_press( button7, button7_pressed, window );

     rect.x = 300; rect.y = 340; rect.w = 200; rect.h = 90;
     res = lite_new_text_button(LITE_BOX(window), &rect, "REBOOT", textButtonTheme3, &button8);
     lite_on_text_button_press( button8, button8_pressed, window );

     rect.x = 550; rect.y = 340; rect.w = 200; rect.h = 90;
     res = lite_new_text_button(LITE_BOX(window), &rect, "EXIT", textButtonTheme3, &button9);
     lite_on_text_button_press( button9, button9_pressed, window );

     /* show the window */
     lite_set_window_background ( window, &white );
     lite_set_window_opacity( window, liteFullWindowOpacity );

     /* run the default event loop */
     lite_window_event_loop( window, 0 );

     /* destroy the window with all this children and resources */
     lite_destroy_window( window );

     lite_destroy_text_button_theme( textButtonTheme1 );
     lite_destroy_text_button_theme( textButtonTheme2 );
     lite_destroy_text_button_theme( textButtonTheme3 );

     /* deinitialize */
     lite_close();

     return 0;
}
